//
//  GENXWriter.h
//
//  Created by Quentin Carnicelli on 10/30/04.
//  Copyright 2004 Rogue Amoeba Software, LLC. All rights reserved.
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to
//  deal in the Software without restriction, including without limitation the
//  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
//  sell copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//  
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
//  IN THE SOFTWARE.

#import <Foundation/Foundation.h>

#import "genx.h"

typedef genxNamespace	GENXNamespace;
typedef genxElement		GENXElement;
typedef genxAttribute	GENXAttribute;

@interface GENXWriter : NSObject
{
	genxWriter		_writer;

	genxSender		_sender;
	NSMutableData*	_data;

	FILE*			_file;
}

+ (NSString*)genxVersion;
+ (NSString*)prefixOfNamespace: (GENXNamespace)ns;

- (id)initForWritingWithMutableData:(NSMutableData*)data;
- (id)initForWritingWithFile: (FILE*)file;

- (NSData*)xmlData;
- (FILE*)xmlFile;

- (void)finishedWriting;

- (GENXNamespace)declareNamespace: (NSString*)prefix withURI: (NSString*)uri;
- (GENXElement)declareElement: (NSString*)name inNamespace: (GENXNamespace)ns;
- (GENXAttribute)delcareAttribute: (NSString*)name inNamespace: (GENXNamespace)ns;

- (void)startDeclaredElement: (GENXElement)e;
- (void)startLiteralElement: (NSString*)name xmlns: (NSString*)xmlns;
- (void)endElement;

- (void)addDeclaredAttribute: (GENXAttribute)attr withValue: (NSString*)val;
- (void)addLiteralAttribute: (NSString*)name withValue: (NSString*)val xmlns: (NSString*)xmlns;

- (void)addText: (NSString*)str;
- (void)addCharacter: (unichar)ch;
- (void)addComment: (NSString*)comment;
- (void)addProcessingInstruction: (NSString*)target text: (NSString*)text;

- (void)addNamespace: (GENXNamespace)ns prefix: (NSString*)prefix;
- (void)unsetDefaultNamespace;

- (BOOL)checkText: (NSString*)str;
- (NSString*)scrubText: (NSString*)str;

@end

@interface GENXWriter (ConvenienceAdditions)
- (void)startLiteralElement: (NSString*)name;
- (void)addLiteralAttribute: (NSString*)name withValue: (NSString*)val;

- (void)addLiteralElement: (NSString*)name withText: (NSString*)text;
@end

__private_extern__ NSString* GENXWriterException;